@@ -1,14 +1,14 @@
 #!/bin/sh
 
 # compile_cdrskin.sh  
-# Copyright 2005 - 2009 Thomas Schmitt, scdbackup@gmx.net, GPL version 2
+# Copyright 2005 - 2010 Thomas Schmitt, scdbackup@gmx.net, GPL
 # to be executed within  ./libburn-*  resp ./cdrskin-*
 
 debug_opts="-O2"
 def_opts=
 largefile_opts="-D_FILE_OFFSET_BITS=64 -D_LARGEFILE_SOURCE=1"
-fifo_opts="-DCdrskin_use_libburn_fifO"
-libvers="-DCdrskin_libburn_0_7_4"
+fifo_opts=""
+libvers="-DCdrskin_libburn_0_7_8"
 
 # To be used if Makefile.am uses libburn_libburn_la_CFLAGS
 # burn="libburn/libburn_libburn_la-"
@@ -20,11 +20,19 @@ libdax_audioxtr_o="$burn"libdax_audioxtr.o
 do_strip=0
 static_opts=
 warn_opts="-Wall"
+libcdio=
 fifo_source="cdrskin/cdrfifo.c"
 compile_cdrskin=1
 compile_cdrfifo=0
 compile_dewav=0
 
+libcam=
+os=$(uname -s)
+case $os in
+ *FreeBSD)
+  libcam="-lcam"
+esac
+
 for i in "$@"
 do
   if test "$i" = "-compile_cdrfifo"
@@ -39,15 +47,15 @@ do
     libdax_audioxtr_o=
     libdax_msgs_o="$burn"message.o
     cleanup_src_or_obj="-DCleanup_has_no_libburn_os_H cdrskin/cleanup.c"
-  elif test "$i" = "-libburn_0_7_4"
+  elif test "$i" = "-libburn_0_7_8"
   then
-    libvers="-DCdrskin_libburn_0_7_4"
+    libvers="-DCdrskin_libburn_0_7_8"
     libdax_audioxtr_o="$burn"libdax_audioxtr.o
     libdax_msgs_o="$burn"libdax_msgs.o
     cleanup_src_or_obj="$burn"cleanup.o
   elif test "$i" = "-libburn_svn"
   then
-    libvers="-DCdrskin_libburn_0_7_5"
+    libvers="-DCdrskin_libburn_0_7_9"
     libdax_audioxtr_o="$burn"libdax_audioxtr.o
     libdax_msgs_o="$burn"libdax_msgs.o
     cleanup_src_or_obj="$burn"cleanup.o
@@ -85,6 +93,13 @@ do
   then
     fifo_source=
     fifo_opts="-DCdrskin_use_libburn_fifO -DCdrskin_no_cdrfifO"
+  elif test "$i" = "-use_libburn_cleanup"
+  then
+    fifo_source=
+    fifo_opts="-DCdrskin_use_libburn_cleanuP -DCdrskin_use_libburn_fifO -DCdrskin_no_cdrfifO"
+  elif test "$i" = "-use_libcdio"
+  then
+    libcdio="-lcdio"
   elif test "$i" = "-g"
   then
     debug_opts="-g"
@@ -94,9 +109,10 @@ do
     echo "Options:"
     echo "  -compile_cdrfifo  compile program cdrskin/cdrfifo."
     echo "  -compile_dewav    compile program test/dewav without libburn."
-    echo "  -libburn_0_7_4    set macro to match libburn-0.7.4"
+    echo "  -libburn_0_7_8    set macro to match libburn-0.7.8"
     echo "  -libburn_svn      set macro to match current libburn-SVN."
     echo "  -dvd_obs_64k      64 KB default size for DVD/BD writing."
+    echo "  -use_libcdio      link with -lcdio because libburn uses it."
     echo "  -do_not_compile_cdrskin  omit compilation of cdrskin/cdrskin."
     echo "  -use_no_libburn_fifo  use cdrfifo even for single track non-CD"
     echo "  -use_no_cdrfifo   always use fifo of libburn and never cdrfifo"
@@ -120,7 +136,7 @@ echo "Build timestamp   :  $timestamp"
 
 if test "$compile_cdrskin"
 then
-  echo "compiling program cdrskin/cdrskin.c $fifo_source $static_opts $debug_opts $libvers $fifo_opts $def_opts $cleanup_src_or_obj"
+  echo "compiling program cdrskin/cdrskin.c $fifo_source $static_opts $debug_opts $libvers $fifo_opts $def_opts $cleanup_src_or_obj $libcdio $libcam"
   cc -I. \
     $warn_opts \
     $static_opts \
@@ -165,6 +181,8 @@ then
     "$burn"crc.o \
     "$burn"ecma130ab.o \
     \
+    $libcdio \
+    $libcam \
     -lpthread
 
     ret=$?
