@@ -1,5 +1,11 @@
 /* -*- indent-tabs-mode: t; tab-width: 8; c-basic-offset: 8; -*- */
 
+/* Copyright (c) 2004 - 2006 Derek Foreman, Ben Jansens
+   Copyright (c) 2006 - 2010 Thomas Schmitt <scdbackup@gmx.net>
+   Provided under GPL version 2 or later.
+*/
+
+
 
 /* <<< ts A91112 : experiments to get better speed with USB
 #define Libburn_sgio_as_growisofS 1
@@ -11,7 +17,7 @@
 This is the main operating system dependent SCSI part of libburn. It implements
 the transport level aspects of SCSI control and command i/o.
 
-Present implementation: Linux SCSI Generic (sg)
+Present implementation: GNU/Linux SCSI Generic (sg)
 
 
 PORTING:
@@ -35,12 +41,28 @@ and of deriving the following system specific files from existing examples:
 Said sg-*.c operations are defined by a public function interface, which has
 to be implemented in a way that provides libburn with the desired services:
  
+sg_id_string()          returns an id string of the SCSI transport adapter.
+                        It may be called before initialization but then may
+                        return only a preliminary id.
+
+sg_initialize()         performs global initialization of the SCSI transport
+                        adapter and eventually needed operating system
+                        facilities. Checks for compatibility of supporting
+                        software components.
+
+sg_shutdown()           performs global finalizations and releases golbally
+                        aquired resources.
+
 sg_give_next_adr()      iterates over the set of potentially useful drive 
                         address strings.
 
 scsi_enumerate_drives() brings all available, not-whitelist-banned, and
                         accessible drives into libburn's list of drives.
 
+sg_dispose_drive()      finalizes adapter specifics of struct burn_drive
+                        on destruction. Releases resources which were aquired
+                        underneath scsi_enumerate_drives().
+
 sg_drive_is_open()      tells wether libburn has the given drive in use.
 
 sg_grab()               opens the drive for SCSI commands and ensures
@@ -55,15 +77,17 @@ sg_issue_command()      sends a SCSI command to the drive, receives reply,
 sg_obtain_scsi_adr()    tries to obtain SCSI address parameters.
 
 
+burn_os_is_2k_seekrw()  tells whether the given path leads to a file object
+                        that can be used in 2 kB granularity by lseek(2) and
+                        read(2), and possibly write(2) if not read-only.
+                        E.g. a USB stick or a hard disk.
+
 burn_os_stdio_capacity()  estimates the emulated media space of stdio-drives.
 
 burn_os_open_track_src()  opens a disk file in a way that allows best
                         throughput with file reading and/or SCSI write command
                         transmission.
 
-burn_os_close_track_src()  closes a filedescriptor obtained by
-                        burn_os_open_track_src().
-
 burn_os_alloc_buffer()  allocates a memory area that is suitable for file
                         descriptors issued by burn_os_open_track_src().
                         The buffer size may be rounded up for alignment
@@ -875,7 +899,7 @@ static int is_scsi_drive(char *fname, int *bus_no, int *host_no,
 #endif /* Libburn_drive_new_deaL */
 
 
-/** Speciality of Linux: detect non-SCSI ATAPI (EIDE) which will from
+/** Speciality of GNU/Linux: detect non-SCSI ATAPI (EIDE) which will from
    then on used used via generic SCSI as is done with (emulated) SCSI drives */ 
 static void ata_enumerate(void)
 {
@@ -1384,7 +1408,7 @@ static void enumerate_common(char *fname, int bus_no, int host_no,
 
 	/* PORTING: ------------------- non portable part --------------- */
 
-	/* Operating system adapter is Linux Generic SCSI (sg) */
+	/* Operating system adapter is GNU/Linux Generic SCSI (sg) */
 	/* Adapter specific handles and data */
 	out.fd = -1337;
 	out.sibling_count = 0;
@@ -1410,8 +1434,63 @@ static void enumerate_common(char *fname, int bus_no, int host_no,
 /* ------------------------------------------------------------------------ */
 
 
+/** Returns the id string  of the SCSI transport adapter and eventually
+    needed operating system facilities.
+    This call is usable even if sg_initialize() was not called yet. In that
+    case a preliminary constant message might be issued if detailed info is
+    not available yet.
+    @param msg   returns id string
+    @param flag  unused yet, submit 0
+    @return      1 = success, <=0 = failure
+*/
+int sg_id_string(char msg[1024], int flag)
+{
+	strcpy(msg, "internal GNU/Linux SG_IO adapter sg-linux");
+	return 1;
+}
+
+
+/** Performs global initialization of the SCSI transport adapter and eventually
+    needed operating system facilities. Checks for compatibility supporting
+    software components.
+    @param msg   returns ids and/or error messages of eventual helpers
+    @param flag  unused yet, submit 0
+    @return      1 = success, <=0 = failure
+*/
+int sg_initialize(char msg[1024], int flag)
+{
+	return sg_id_string(msg, 0);
+}
+
+
+/** Performs global finalization of the SCSI transport adapter and eventually
+    needed operating system facilities. Releases globally aquired resources.
+    @param flag  unused yet, submit 0
+    @return      1 = success, <=0 = failure
+*/  
+int sg_shutdown(int flag)
+{
+	return 1;
+}
+
+
+/** Finalizes BURN_OS_TRANSPORT_DRIVE_ELEMENTS, the components of
+    struct burn_drive which are defined in os-*.h.
+    The eventual initialization of those components was made underneath
+    scsi_enumerate_drives().
+    This will be called when a burn_drive gets disposed.
+    @param d     the drive to be finalized
+    @param flag  unused yet, submit 0
+    @return      1 = success, <=0 = failure
+*/
+int sg_dispose_drive(struct burn_drive *d, int flag)
+{
+        return 1;
+}
+
+
 /** PORTING:
-    In this Linux implementation, this function mirrors the enumeration
+    In this GNU/Linux implementation, this function mirrors the enumeration
     done in sg_enumerate and ata_enumerate(). It would be better to base those
     functions on this sg_give_next_adr() but the situation is not inviting. 
 */
@@ -1508,7 +1587,7 @@ return_1_pre_proc:;
     libburn's list of drives.
 */
 /** PORTING:
-    If not stricken with an incompletely unified situation like in Linux
+    If not stricken with an incompletely unified situation like in GNU/Linux
     one would rather implement this by a loop calling sg_give_next_adr().
     If needed with your sg_give_next_adr() results, do a test for existence
     and accessability. If burn activities are prone to external interference
@@ -1572,6 +1651,7 @@ int sg_drive_is_open(struct burn_drive * d)
 int sg_grab(struct burn_drive *d)
 {
 	int fd, os_errno= 0, ret;
+	int max_tries = 3, tries = 0;
 
 	/* ts A60813 */
 	int open_mode = O_RDWR;
@@ -1605,6 +1685,7 @@ int sg_grab(struct burn_drive *d)
 	   value -1 of open(2) war used. */
 	if(! burn_drive_is_open(d)) {
 
+try_open:;
 		/* ts A60821
    		<<< debug: for tracing calls which might use open drive fds */
 		mmc_function_spy(NULL, "sg_grab ----------- opening");
@@ -1638,13 +1719,19 @@ int sg_grab(struct burn_drive *d)
 		fcntl(fd, F_SETOWN, getpid());
 		d->released = 0;
 		return 1;
-	}
+	} else if (errno == EBUSY)
+		goto drive_is_in_use;
 	libdax_msgs_submit(libdax_messenger, d->global_index, 0x00020003,
 			LIBDAX_MSGS_SEV_SORRY, LIBDAX_MSGS_PRIO_HIGH,
 			"Could not grab drive", os_errno, 0);
 	return 0;
 
 drive_is_in_use:;
+	tries++;
+	if (tries < max_tries) {
+		usleep(2000000);
+		goto try_open;
+	}
 	libdax_msgs_submit(libdax_messenger, d->global_index,
 			0x00020003,
 			LIBDAX_MSGS_SEV_SORRY, LIBDAX_MSGS_PRIO_HIGH,
@@ -1682,43 +1769,8 @@ int sg_release(struct burn_drive *d)
 }
 
 
-/* <<< ts A91111: on its way out */
-/** ts A70518: 
-    Debugging log facility. Controlled by existence of macros:
-     Libburn_log_sg_commandS          enables logging to file
-                                        /tmp/libburn_sg_command_log
-     Libburn_fflush_log_sg_commandS   enables fflush after each output line
-     Libburn_log_sg_command_stderR    enables additional log to stderr
-*/
-/*
- ts A91111: now enabled by default and controlled burn_sg_log_scsi
-*/
-#define Libburn_log_sg_commandS 1
-#define Libburn_fflush_log_sg_commandS 1
-#define Libburn_log_sg_command_stderR 1
-
-
-#ifdef Libburn_log_sg_commandS
-
-/** Logs command (before execution) */
-static int sg_log_cmd(struct command *c, FILE *fp, int flag)
-{
-	if (fp != NULL && (fp == stderr || (burn_sg_log_scsi & 1))) {
-		scsi_show_cmd_text(c, fp, 0);
-
-#ifdef Libburn_fflush_log_sg_commandS
-		if (burn_sg_log_scsi & 4)
-			fflush(fp);
-#endif
-	}
-#ifdef Libburn_log_sg_command_stderR
-	if (fp == stderr || !(burn_sg_log_scsi & 2))
-		return 1;
-	sg_log_cmd(c, stderr, flag);
-#endif
-	return 1;
-}
-
+#ifdef NIX
+/* <<< now in spc.c as scsi_log_err */
 
 /** logs outcome of a sg command. flag&1 causes an error message */
 static int sg_log_err(struct command *c, FILE *fp, 
@@ -1734,21 +1786,17 @@ static int sg_log_err(struct command *c, FILE *fp,
 			scsi_show_cmd_reply(c, fp, 0);
 			fprintf(fp,"%6d ms\n", s->duration);
 		}
-#ifdef Libburn_fflush_log_sg_commandS
 		if (burn_sg_log_scsi & 4)
 			fflush(fp);
-#endif
 	}
-#ifdef Libburn_log_sg_command_stderR
 	if (fp == stderr || !(burn_sg_log_scsi & 2))
 		return 1;
+
 	sg_log_err(c, stderr, s, flag);
-#endif
 	return 1;
 }
 
-
-#endif /* Libburn_log_sg_commandS */
+#endif /* NIX */
 
 
 /** Sends a SCSI command to the drive, receives reply and evaluates wether
@@ -1756,8 +1804,6 @@ static int sg_log_err(struct command *c, FILE *fp,
     Returned SCSI errors shall not lead to a return value indicating failure.
     The callers get notified by c->error. An SCSI failure which leads not to
     a retry shall be notified via scsi_notify_error().
-    The Libburn_log_sg_commandS facility might be of help when problems with
-    a drive have to be examined. It shall stay disabled for normal use.
     @return: 1 success , <=0 failure
 */
 int sg_issue_command(struct burn_drive *d, struct command *c)
@@ -1766,11 +1812,8 @@ int sg_issue_command(struct burn_drive *d, struct command *c)
 	int err;
 	time_t start_time;
 	sg_io_hdr_t s;
-
-#ifdef Libburn_log_sg_commandS
 	/* ts A61030 */
 	static FILE *fp= NULL;
-#endif /* Libburn_log_sg_commandS */
 
 	/* <<< ts A60821
 	   debug: for tracing calls which might use open drive fds */
@@ -1779,7 +1822,6 @@ int sg_issue_command(struct burn_drive *d, struct command *c)
 		d->fd,d->released);
 	mmc_function_spy(NULL, buf);
 
-#ifdef Libburn_log_sg_commandS
 	/* ts A61030 */
 	if (burn_sg_log_scsi & 1) {
 		if (fp == NULL) {
@@ -1789,9 +1831,7 @@ int sg_issue_command(struct burn_drive *d, struct command *c)
 		}
 	}
 	if (burn_sg_log_scsi & 3)
-		sg_log_cmd(c,fp,0);
-#endif /* Libburn_log_sg_commandS */
-	  
+		scsi_log_cmd(c,fp,0);
 
 	/* ts A61010 : with no fd there is no chance to send an ioctl */
 	if (d->fd < 0) {
@@ -1931,12 +1971,9 @@ ex:;
 				LIBDAX_MSGS_SEV_DEBUG, LIBDAX_MSGS_PRIO_HIGH,
 				msg, 0, 0);
 	}
-
-#ifdef Libburn_log_sg_commandS
 	if (burn_sg_log_scsi & 3)
-		sg_log_err(c, fp, &s, c->error != 0);
-#endif /* Libburn_log_sg_commandS */
-
+		/* <<< sg_log_err(c, fp, &s, c->error != 0); */
+		scsi_log_err(c, fp, s.sbp, s.duration, c->error != 0);
 	return 1;
 }
 
@@ -2032,6 +2069,24 @@ int sg_is_enumerable_adr(char *adr)
 }
 
 
+/* ts B00115 */
+/* Return 1 if the given path leads to a regular file or a device that can be
+   seeked, read, and possibly written with 2 kB granularity. 
+*/
+int burn_os_is_2k_seekrw(char *path, int flag)
+{
+	struct stat stbuf;
+
+	if (stat(path, &stbuf) == -1)
+		return 0;
+	if (S_ISREG(stbuf.st_mode))
+		return 1;
+	if (S_ISBLK(stbuf.st_mode))
+		return 1;
+	return 0;
+}
+
+
 /* ts A70909 */
 /** Estimate the potential payload capacity of a file address.
     @param path  The address of the file to be examined. If it does not
@@ -2083,7 +2138,7 @@ int burn_os_stdio_capacity(char *path, off_t *bytes)
 	if (testpath[0]) {	
 		if (statvfs(testpath, &vfsbuf) == -1)
 			return -2;
-		*bytes = add_size + ((off_t) vfsbuf.f_bsize) *
+		*bytes = add_size + ((off_t) vfsbuf.f_frsize) *
 						(off_t) vfsbuf.f_bavail;
 	}
 	return 1;
