@@ -1,7 +1,7 @@
 
 /* test/libburner.c , API illustration of burning data or audio tracks to CD */
-/* Copyright (C) 2005 - 2009 Thomas Schmitt <scdbackup@gmx.net> */
-/* Provided under GPLv2,see also "License and copyright aspects" at file end */
+/* Copyright (C) 2005 - 2010 Thomas Schmitt <scdbackup@gmx.net> */
+/* Provided under GPL, see also "License and copyright aspects" at file end */
 
 
 /**                               Overview 
@@ -26,7 +26,7 @@
   approaches are shown here in application functions:
      libburner_aquire_by_adr()     demonstrates usage as of cdrecord traditions
      libburner_aquire_by_driveno()      demonstrates a scan-and-choose approach
-  With that aquired drive you can blank a CD-RW
+  With that aquired drive you can blank a CD-RW or DVD-RW
      libburner_blank_disc()
   or you can format a DVD-RW to profile "Restricted Overwrite" (needed once)
   or an unused BD to default size with spare blocks
@@ -36,9 +36,25 @@
   When everything is done, main() releases the drive and shuts down libburn:
      burn_drive_release();
      burn_finish()
+
+  FreeBSD does not work well with the convenient synchronous signal handler. So
+  the waiting loops for blanking, formatting, and writing use the asynchronous
+  mode of the libburn signal handler. It will not shutdown the library and
+  abort the program, but rather tell the ongoing drive operation to stop as
+  soon as possible. After the loops and at the end of the program there is a
+  call to determine whether an abort happened:
+     burn_is_aborting()
   
+  Applications must use 64 bit off_t. E.g. by defining
+    #define _LARGEFILE_SOURCE
+    #define _FILE_OFFSET_BITS 64
+  or take special precautions to interface with the library by 64 bit integers
+  where libburn/libburn.h prescribes off_t.
+  This program gets fed with appropriate settings externally by libburn's
+  autotools generated build system.
 */
 
+
 /** See this for the decisive API specs . libburn.h is The Original */
 /*  For using the installed header file :  #include <libburn/libburn.h> */
 /*  This program insists in the own headerfile. */
@@ -300,9 +316,12 @@ int libburner_blank_disc(struct burn_drive *drive, int blank_fast)
 			"FATAL : Media is not of erasable type\n");
 		return 0;
 	}
-	printf(
-	      "Beginning to %s-blank media.\n", (blank_fast?"fast":"full"));
+	/* Switch to asynchronous signal handling for the time of waiting */
+	burn_set_signal_handling("libburner : ", NULL, 0x30);
+
+	printf("Beginning to %s-blank media.\n", (blank_fast?"fast":"full"));
 	burn_disc_erase(drive, blank_fast);
+
 	sleep(1);
 	while (burn_drive_get_status(drive, &p) != BURN_DRIVE_IDLE) {
 		if(p.sectors>0 && p.sector>=0) /* display 1 to 99 percent */
@@ -311,6 +330,10 @@ int libburner_blank_disc(struct burn_drive *drive, int blank_fast)
 		printf("Blanking  ( %.1f%% done )\n", percent);
 		sleep(1);
 	}
+	if (burn_is_aborting(0) > 0)
+		return -1;
+	/* Back to synchronous handling */
+	burn_set_signal_handling("libburner : ", NULL, 0);
 	printf("Done\n");
 	return 1;
 }
@@ -363,6 +386,7 @@ int libburner_format(struct burn_drive *drive)
 		fprintf(stderr, "FATAL: Can only format DVD-RW or BD\n");
 		return 0;
 	}
+	burn_set_signal_handling("libburner : ", NULL, 0x30);
 
 	printf("Beginning to format media.\n");
 	burn_disc_format(drive, size, format_flag);
@@ -375,6 +399,9 @@ int libburner_format(struct burn_drive *drive)
 		printf("Formatting  ( %.1f%% done )\n", percent);
 		sleep(1);
 	}
+	if (burn_is_aborting(0) > 0)
+		return -1;
+	burn_set_signal_handling("libburner : ", NULL, 0);
 	burn_disc_get_profile(drive_list[0].drive, &current_profile,
 				 current_profile_name);
 	if (current_profile == 0x14 || current_profile == 0x13)
@@ -520,6 +547,7 @@ int libburner_payload(struct burn_drive *drive,
 		fprintf(stderr, "Reasons given:\n%s\n", reasons);
 		return 0;
 	}
+	burn_set_signal_handling("libburner : ", NULL, 0x30);
 
 	printf("Burning starts. With e.g. 4x media expect up to a minute of zero progress.\n");
 	start_time = time(0);
@@ -567,6 +595,8 @@ int libburner_payload(struct burn_drive *drive,
 	}
 	burn_session_free(session);
 	burn_disc_free(target_disc);
+	if (burn_is_aborting(0) > 0)
+		return -1;
 	if (multi && current_profile != 0x1a && current_profile != 0x13 &&
 		current_profile != 0x12 && current_profile != 0x43) 
 			/* not with DVD+RW, formatted DVD-RW, DVD-RAM, BD-RE */
@@ -718,7 +748,7 @@ int main(int argc, char **argv)
 	/* Print messages of severity SORRY or more directly to stderr */
 	burn_msgs_set_severities("NEVER", "SORRY", "libburner : ");
 
-	/* Activate the default signal handler which eventually will try to
+	/* Activate the synchronous signal handler which eventually will try to
 	   properly shutdown drive and library on aborting events. */
 	burn_set_signal_handling("libburner : ", NULL, 0);
 
@@ -752,10 +782,14 @@ release_drive:;
 		burn_drive_release(drive_list[driveno].drive, 0);
 
 finish_libburn:;
+	if (burn_is_aborting(0) > 0) {
+		burn_abort(4400, burn_abort_pacifier, "libburner : ");
+		fprintf(stderr,"\nlibburner run aborted\n");
+		exit(1);
+	} 
 	/* This app does not bother to know about exact scan state. 
 	   Better to accept a memory leak here. We are done anyway. */
 	/* burn_drive_info_free(drive_list); */
-
 	burn_finish();
 	exit(ret);
 }
@@ -768,37 +802,6 @@ Read. Try. Think. Play. Write yourself some code. Be free of my copyright.
 
 Be also invited to study the code of cdrskin/cdrskin.c et al.
 
-
-Clarification in my name and in the name of Mario Danic, copyright holder
-on toplevel of libburnia. To be fully in effect after the remaining other
-copyrighted code has been replaced by ours and by copyright-free contributions
-of our friends:
-
-We, the copyright holders, agree on the interpretation that
-dynamical linking of our libraries constitutes "use of" and
-not "derivation from" our work in the sense of GPL, provided
-those libraries are compiled from our unaltered code.
-
-Thus you may link our libraries dynamically with applications
-which are not under GPL. You may distribute our libraries and
-application tools in binary form, if you fulfill the usual
-condition of GPL to offer a copy of the source code -altered
-or unaltered- under GPL.
-
-We ask you politely to use our work in open source spirit
-and with the due reference to the entire open source community.
-
-If there should really arise the case where above clarification
-does not suffice to fulfill a clear and neat request in open source
-spirit that would otherwise be declined for mere formal reasons,
-only in that case we will duely consider to issue a special license
-covering only that special case.
-It is the open source idea of responsible freedom which will be
-decisive and you will have to prove that you exhausted all own
-means to qualify for GPL.
-
-For now we are firmly committed to maintain one single license: GPL.
-
 History:
 libburner is a compilation of my own contributions to test/burniso.c and
 fresh code which replaced the remaining parts under copyright of
